// Generated by CoffeeScript 1.12.5

/*
Copyright 2016 Resin.io

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */
var _, bash, getSubShellCommand;

_ = require('lodash');

bash = require('bash');

getSubShellCommand = require('../utils/helpers').getSubShellCommand;

module.exports = {
  signature: 'ssh [uuid]',
  description: '(beta) get a shell into the running app container of a device',
  help: 'Warning: \'resin ssh\' requires an openssh-compatible client to be correctly\ninstalled in your shell environment. For more information (including Windows\nsupport) please check the README here: https://github.com/resin-io/resin-cli\n\nUse this command to get a shell into the running application container of\nyour device.\n\nExamples:\n\n	$ resin ssh MyApp\n	$ resin ssh 7cf02a6\n	$ resin ssh 7cf02a6 --port 8080\n	$ resin ssh 7cf02a6 -v',
  permission: 'user',
  primary: true,
  options: [
    {
      signature: 'port',
      parameter: 'port',
      description: 'ssh gateway port',
      alias: 'p'
    }, {
      signature: 'verbose',
      boolean: true,
      description: 'increase verbosity',
      alias: 'v'
    }, {
      signature: 'noproxy',
      boolean: true,
      description: "don't use the proxy configuration for this connection. Only makes sense if you've configured proxy globally."
    }
  ],
  action: function(params, options, done) {
    var Promise, child_process, patterns, resin, verbose;
    child_process = require('child_process');
    Promise = require('bluebird');
    resin = require('resin-sdk-preconfigured');
    patterns = require('../utils/patterns');
    if (options.port == null) {
      options.port = 22;
    }
    verbose = options.verbose ? '-vvv' : '';
    return Promise["try"](function() {
      if (!params.uuid) {
        return false;
      }
      return resin.models.device.has(params.uuid);
    }).then(function(uuidExists) {
      if (uuidExists) {
        return params.uuid;
      }
      return patterns.inferOrSelectDevice();
    }).then(function(uuid) {
      console.info("Connecting to: " + uuid);
      return resin.models.device.get(uuid);
    }).then(function(device) {
      if (!device.is_online) {
        throw new Error('Device is not online');
      }
      return Promise.props({
        username: resin.auth.whoami(),
        uuid: device.uuid,
        containerId: resin.models.device.getApplicationInfo(device.uuid).get('containerId'),
        proxyUrl: resin.settings.get('proxyUrl')
      }).then(function(arg) {
        var containerId, proxyUrl, username, uuid;
        username = arg.username, uuid = arg.uuid, containerId = arg.containerId, proxyUrl = arg.proxyUrl;
        if (containerId == null) {
          throw new Error('Did not find running application container');
        }
        return Promise["try"](function() {
          var command, i, proxyAuth, proxyCommand, proxyConfig, sshProxyCommand, subShellCommand, tunnelOptions;
          sshProxyCommand = '';
          proxyConfig = global.PROXY_CONFIG;
          if (proxyConfig && !options.noproxy) {
            tunnelOptions = {
              proxy: proxyConfig.host + ":" + proxyConfig.port,
              dest: '%h:%p'
            };
            proxyAuth = proxyConfig.proxyAuth;
            if (proxyAuth) {
              i = proxyAuth.indexOf(':');
              _.assign(tunnelOptions, {
                user: proxyAuth.substring(0, i),
                pass: proxyAuth.substring(i + 1)
              });
            }
            proxyCommand = "proxytunnel " + (bash.args(tunnelOptions, '--', '='));
            sshProxyCommand = "-o " + (bash.args({
              ProxyCommand: proxyCommand
            }, '', '='));
          }
          command = "ssh " + verbose + " -t -o LogLevel=ERROR -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null -o ControlMaster=no " + sshProxyCommand + " -p " + options.port + " " + username + "@ssh." + proxyUrl + " enter " + uuid + " " + containerId;
          subShellCommand = getSubShellCommand(command);
          return child_process.spawn(subShellCommand.program, subShellCommand.args, {
            stdio: 'inherit'
          });
        });
      });
    }).nodeify(done);
  }
};
